/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __BASE_MODULE_H__
#define __BASE_MODULE_H__

#include <QObject>
#include <QColor>
#include <QString>


#include "modulemanager.h"

#define QS(str) ((char*)str.toUtf8().data())

class QProcess;


namespace Tuxtalador2{

	/**
	 * @short A tuxtalador install unit, get some data, do something at install.
	 *
 	 * At constructor time each module must register 
	 */
	class BaseModule : public QObject{
		Q_OBJECT
	public:
		BaseModule(ModuleManager *parent): 
			QObject(parent),manager(parent){
			connect(this,SIGNAL(log(const QString &,QColor)),
					parent, SIGNAL(log(const QString &,QColor)));
		}

		/// Must return the widget to show at the wizard at that step, or 
		/// NULL if just at a install step.
		virtual QWidget *wizard(int step){ return NULL; };

		/// Title/label for this step.
		virtual QString label(int step){ return tr("Unknown"); };
		
		/// When at a new page, checks if it can continue.
		virtual bool canContinueNow(int step){ return true; };
		/// When at a new page, checks if it can go back.
		virtual bool canGoBack(int step){ return true; };

		/// Returns current module manager.
		ModuleManager *moduleManager(){ return manager; };

	signals:
		/// At installation, the % of the progress.
		void progress(int progress);
		/// Add a message to the log pane
		void log(const QString &logmsg, QColor color=Qt::black);
		/// The current step finished (install, or wizard, go to the next).
		/// It is of vital importance to call this when finished.
		void endStep();
		/// Change the 'next' button status
		void canContinue(bool);

	protected:
		ModuleManager *manager;
	};
};

#endif
