/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>

#include "moduleloader.h"
#include "basemodule.h"


/// Base modules
#include "modules/welcome.h"
#include "modules/userdata.h"
#include "modules/diskedit.h"
#include "modules/baseinstall.h"
#include "modules/install.h"
#include "modules/postinstall.h"

using namespace Tuxtalador2;

/**
 * @short Loads all the modules that the system needs.
 */
void ModuleLoader::loadModules(ModuleManager *moduleManager){
	bool OEM=!moduleManager->getConfig("OEM").isEmpty();
	bool firstBoot=!moduleManager->getConfig("firstBoot").isEmpty();
	
	if (!OEM){	
		new Welcome(moduleManager);
		new Userdata(moduleManager);
		new DiskEdit(moduleManager);
		new BaseInstall(moduleManager);
		new Install(moduleManager);
		new PostInstall(moduleManager);
	}
	else{
		if (!firstBoot){ // Install to HD
			new Welcome(moduleManager);
			new DiskEdit(moduleManager);
			new BaseInstall(moduleManager);
			new Install(moduleManager);
			new PostInstall(moduleManager);
		}
		else{ // first boot
			new Welcome(moduleManager);
			new Userdata(moduleManager);
			new PostInstall(moduleManager);
		}
	}
}





