/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MODULE_MANAGER_H__
#define __MODULE_MANAGER_H__

#include <QObject>
#include <QColor>
#include <QMap>

namespace Tuxtalador2{
	
	class BaseModule;

	/**
	 * @short Manages modules: At instantiate, wizard and install times.
	 *
	 * Modules add themselfs to this manager with addWizard/addInstaller.
	 * Both need the Module pointer, and where to install it. The place is an
	 * integer. Several things have a given place:
	 *
	 * 1. Introduction
	 * 2. License
	 * 50. Install
	 * 100. Good bye.
	 *
	 * the best is to look at MODULES.txt to have a list with order. If you
	 * change anything, please keep all data in sync.
	 *
	 * It also contains a public dictionary that is used to store temporal
	 * config data that may be needed by other modules (like target, 
	 * cdrom_drive...)
	 */
	class ModuleManager : public QObject{
		Q_OBJECT
	public:
		/// Resume of the modules and the positions. May appear several times.
		struct ModulePosition{
			ModulePosition(BaseModule *mod, int pos){ module=mod; position=pos; };

			int position;
			BaseModule *module;
		};

		ModuleManager(QObject *parent=NULL);

		bool addWizard(BaseModule *module, int where);

		QString getConfig(const QString &key);
		QStringList getConfigKeys();
		void setConfig(const QString &key, const QString &value);

		/// Returns the module positions lists. Usefull for menu.
		QList<ModulePosition> getModules(){ return wizards; }

		bool atLastStep();
	signals:
		/// At installation, the % of the progress.
		void progress(int progress);
		/// Add a message to the log pane
		void log(const QString &logmsg, QColor color=Qt::black);
		/// The current step finished (install, or wizard, go to the next).
		/// It is of vital importance to call this when finished.
		void endStep(int step);
		/// Advises to set this new wizard.
		void setWizard(BaseModule *wizard, int position);

	public slots:
		void goPrevious();
		void goNext();
		bool go(int);
		void partialProgress(int percent);
		void writeLog(const QString &str, QColor color=Qt::black){ emit log(str,color); }

	protected slots:
		void endStepSlot();

	protected:
		QList<ModulePosition> wizards;
		int step;
		int nextStep;

		QMap<QString,QString> globalConfig;
	};
};

#endif
