/*
TRANSLATOR Tuxtalador2::DiskEdit
*/
/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QTreeView>
#include <QDir>
#include <QMessageBox>
#include <QHeaderView>
#include <QRegExp>
#include <QPushButton>
#include <QApplication>
#include <QCheckBox>
#include <QVBoxLayout>
#include <QMessageBox>

#include <unistd.h>

#include "../process.h"
#include "diskedit.h"
#include <QInputDialog>

using namespace Tuxtalador2;

DiskEdit::DiskEdit(ModuleManager *mm):BaseModule(mm){
	mm->addWizard(this,15);
	mm->addWizard(this,25);
	mm->addWizard(this,30);

	mm->addWizard(this,95);
	//mm->addWizard(this,98);
	diskModel = new DiskModel;
	mounted=false;
}

DiskEdit::~DiskEdit(){
	if (mounted){
		system("umount /target");
	}
}

/// Widgets for each step. The model is kept, but not the visualization.
QWidget *DiskEdit::wizard(int step){
	switch(step){
	case 15:
	{
		QWidget *root=new QWidget();
		
		QGridLayout *layout=new QGridLayout(root);
		root->setLayout(layout);
	
		QLabel *label=new QLabel(tr("<h1>Please select the partition to use to install %1 to.</h1>\n"
									"You can choose a partition or a disk. If you choose a disk, a\n"
									"partition table is created.").
									arg(manager->getConfig("OSDESCRIPTION")),root);
		label->setWordWrap(true);
		layout->addWidget(label,0,0,1,5);

		treeView=new QTreeView(root);
		layout->addWidget(treeView,1,0,1,5);
 		
		treeView->setModel(diskModel);
		treeView->expandAll();
		treeView->header()->resizeSections(QHeaderView::ResizeToContents);

		/*
		QCheckBox *showAdvanced=new QCheckBox(tr("Show advanced options"),root);
		layout->addWidget(showAdvanced, 2,0);

		QWidget *advanced=new QWidget(root);
		connect(showAdvanced,SIGNAL(stateChanged(int)),this,SLOT(showAdvanced(int)));
		QVBoxLayout *advancedLayout=new QVBoxLayout(advanced);

		QCheckBox *useRAID=new QCheckBox(tr("Use RAID 1 (mirrored)"), advanced);
		advancedLayout->addWidget(useRAID);

		layout->addWidget(advanced, 3,0,2,5);
		*/

		QPushButton *partitionmanager=new QPushButton(tr("Open Partition Manager"), root);
		layout->addWidget(partitionmanager, 4,0,2,5);
		connect(partitionmanager,SIGNAL(clicked()),
				this, SLOT(openPartitionManager()));

		connect(treeView, SIGNAL(pressed(const QModelIndex &)),
				this, SLOT(selectPartition(const QModelIndex &)));
		connect(treeView, SIGNAL(activated(const QModelIndex &)),
				this, SLOT(selectPartition(const QModelIndex &)));

		return root;
	}
	break;
	case 25:
	{
		emit log(QString("Create target"));
		if (!diskModel->ensureRootValid()){
			QMessageBox::critical(NULL, tr("Not a valid partition selected"),
								   tr("Not a valid partition selected"));
			manager->goPrevious();
		}
		QDir::root().mkdir("/target/");
		emit log(QString("Format %1").arg(diskModel->getRootPartition()));
		manager->setConfig("ROOT_DEVICE",diskModel->getRootPartition());
		
		QFile procfstab("/proc/fstab");
		procfstab.open(QFile::ReadOnly);
		if (QString(procfstab.read(1024)).contains(diskModel->getRootPartition())){
			QMessageBox::warning(NULL, tr("Partition mounted"),
										tr("This partition (%1) is already mounted.\n"
										   "Please check you really want to format it, and try again"
										   ).arg(diskModel->getRootPartition()));
			manager->goPrevious();
		}

		QProcess::execute("mdev", QStringList("-s"));
		
		process=new Process(QString("mkfs.ext3 %1").arg(diskModel->getRootPartition()),
						this,false,true);
		connect(process,SIGNAL(log(const QString&,QColor)),
				this,SLOT(readLog(const QString&, QColor)));
		return NULL;
	}
	break;
	case 30:
		process=new Process(QString("mount -t ext3 %1 /target/").arg(diskModel->getRootPartition()),
						this,false,true);
		mounted=true;
		return NULL;
	break;
	case 95:
	{
		// Maybe not the best place, but I have to do it somewhere, and if it fails, it does not harm.
		system("chroot /target umount proc");
		system("chroot /target umount sys");
		system("chroot /target umount sys/fs/fuse/connections");
		system("umount /target/cdrom");
		system("fuser -k /target/"); // Not elegant at all.. but necesary, to be able to umount.

		
		int ok=system(QString("umount %1").arg(diskModel->getRootPartition()).toUtf8().data());

		if (ok!=0){
			manager->writeLog(tr("Error umounting. (0)"),QColor("red"));
			qApp->processEvents();
			sleep(1);
			manager->writeLog(tr("Error umounting. (1)"),QColor("red"));
			qApp->processEvents();
			system("chroot /target umount proc");
			system("chroot /target umount sys");
			system("umount /target/cdrom");
			system("fuser -k /target/"); // Not elegant at all.. but necesary, to be able to umount.
			ok=system(QString("umount %1").arg(diskModel->getRootPartition()).toUtf8().data());
			if(ok!=0){
				manager->writeLog(tr("Error umounting. Continuing anyway with just a sync."),QColor("red"));
				qApp->processEvents();
				system("sync");
			}
			else{
				manager->writeLog(tr("Umount success."),QColor("green"));
			}
		}
		if (ok==0){
			mounted=false;
			manager->goNext();
		}
		return NULL;
	}
	break;
	}
	return NULL;
}

/// Labels for each step
QString DiskEdit::label(int step){
	switch(step){
	case 15:
		return tr("Select install partition");
	case 25:
		return tr("Prepare partition");
	case 30:
		return tr("Mounting partition");
	case 95:
		return tr("Umounting partition");
	}
	return QString::null;
}

/// Can continue if selected root and at first step.
bool DiskEdit::canContinueNow(int step){
	if (step==15)
		return !diskModel->getRootPartition().isEmpty();
	return false;
}


/**
 * @short If in basic mode, then just set as root.
 */
void DiskEdit::selectPartition(const QModelIndex &index){
	diskModel->cleanFormatsAndMounts();

	diskModel->setFormat(index);
	diskModel->setMountPoint(index,"/");
	QString rootp=diskModel->getRootPartition();
	manager->setConfig("ROOT_DEVICE",rootp);

	emit canContinue(!rootp.isEmpty());
}

void DiskEdit::readLog(const QString &str,QColor){
	QRegExp re("(\\d+)/(\\d+)");
	if (re.indexIn(str)>=0){
		int a=re.cap(1).toInt();
		int b=re.cap(2).toInt();
		//qDebug("%s:%d %d %d %p",__FILE__,__LINE__,a,b,manager);
		if (b>0) // no div by zero, please.
			manager->partialProgress(a*120/b);
	}
}

/**
 * @short Opens the partition manager
 */
void DiskEdit::openPartitionManager(){
	Process *p=new Process("gpartedbin", this, false, false);
	connect(p,SIGNAL(finished(int,QProcess::ExitStatus)),
			this,SLOT(reloadPartitionData()));
	//connect(p,SIGNAL(finished(int,QProcess::ExitStatus)),
	//		p,SLOT(deleteLater()));
}

/**
 * @short Reinitializes model data
 */
void DiskEdit::reloadPartitionData(){
	diskModel->initialize();
	manager->writeLog(tr("Reloading partitions data"));
	treeView->expandAll();
}


/**
 * @short Just calls the initialization method
 */
DiskModel::DiskModel(){
 	initialize();
}

/// Reinitializes the data for the model.
void DiskModel::initialize(){
	rootpartition=NULL;

	foreach(Node *n, disks){
		foreach(Node *p, n->partitions){
			delete p;
		}
		delete n;
	}
	disks.clear();

	ped_device_probe_all();

	int ndisks=0, npartitions=0, npartitions_lastdisk=0;
	
	PedDevice *nextdevice=NULL;
	Node *d;
	while ((nextdevice=ped_device_get_next(nextdevice))!=NULL){
		if (nextdevice->read_only) // quite probably a CDROM drive, anyway it could not use for installation if ro.
			continue; 
		PedDisk *nextdisk=ped_disk_new(nextdevice);
		if (!nextdisk){ // no partition table!
			int answer;
			answer=QMessageBox::question(NULL, tr("No partition table found on %1").arg(nextdevice->path),
							tr("No partitions found on %1.\nMaybe it needs a partition table.\n"
								"Do you want to create a new partition table on the device?\n").
							arg(nextdevice->path),
							QMessageBox::Yes|QMessageBox::No);
			if (answer==QMessageBox::Yes)
				nextdisk=ped_disk_new_fresh(nextdevice,ped_disk_type_get("msdos"));
			else
				continue;
		}
		
		d=new Node;
		d->disk=nextdisk;
		d->partition=NULL;
		d->parent=NULL;
		d->format=false;

		npartitions_lastdisk=0;
		
		PedPartition *nextpartition=NULL;
		while ((nextpartition=ped_disk_next_partition(nextdisk,nextpartition))!=NULL){

			if ( nextpartition->type!=PED_PARTITION_NORMAL &&
				nextpartition->type!=PED_PARTITION_LOGICAL &&
				nextpartition->type!=PED_PARTITION_FREESPACE )
				continue;
			

			//if (nextpartition->num<0)
			//	continue;
			Node *p=new Node;
			p->disk=nextdisk;
			p->partition=nextpartition;
			p->parent=d;
			d->format=false;

			d->partitions.append(p);
			npartitions++;
		}
		
		disks.append(d);
		ndisks++;
	}
	qDebug("%s:%d %d disks, %d partitions",__FILE__,__LINE__,ndisks,npartitions);
	reset();
}

/// Converts from sectors to a Human Readable (but in binary prefixes)
QString toHumanSize(double size){
	QString suffix[]={"B","kiB","MiB","GiB","TiB"};
	int n=0;
	double dsize=size*512;
	while (dsize>1024.0){
		dsize=dsize/1024.0;
		n+=1;
	}
	return QString("%1 %2").arg(dsize,5,'f',2).arg(suffix[n]);
}

/// Data to show at each disk/partition and column and role.
QVariant DiskModel::data(const QModelIndex &index, int role) const{
	if (index.isValid()){
		if (role==Qt::DisplayRole){
			Node *node=NODE_FROM_INDEX(index);
			if (!node || !node->disk)
				return ".";
			switch(index.column()){
			case 0:
				if (node->partition){
					switch (node->partition->type){
						case PED_PARTITION_NORMAL:
						case PED_PARTITION_LOGICAL:
							return QString("%1%2").arg(node->disk->dev->path).arg(node->partition->num);
						break;
						case PED_PARTITION_FREESPACE:
							return tr("Freespace");
						break;
						default:
							return tr("Unkown");
						break;
					}
				}
				Q_ASSERT(node);
				Q_ASSERT(node->disk);
				Q_ASSERT(node->disk->dev);
				Q_ASSERT(node->disk->dev->path);
				return node->disk->dev->path;
			case 1:
				if (node->partition)
					return "";
				return node->disk->dev->model;
			case 2:
				if (node->partition){
					unsigned long size=node->partition->geom.end-node->partition->geom.start;
					return toHumanSize(size);
				}
				return toHumanSize(node->disk->dev->hw_geom.sectors*
								   node->disk->dev->hw_geom.heads*
								   node->disk->dev->hw_geom.cylinders);
			case 3:
				if (node->partition)
					return (node->partition->fs_type) ? node->partition->fs_type->name : tr("none");
				{
				QString disk=QString(node->disk->dev->path).section('/',-1,-1);
				if (disk.startsWith("sd"))
					return tr("SCSI Disk");
				else if (disk.startsWith("hd"))
					return tr("IDE Disk");
				return tr("Disk");
				}
			case 4:
				return node->mountPoint;
			case 5:
				return node->format ? (node->partition ? tr("Format!") : tr("Partition & format!") ) : "";
			default:
				return QVariant();
			}
		}
	}
	return QVariant();
}

/// List of header names.
QVariant DiskModel::headerData(int section, Qt::Orientation orientation,
					int role) const{
	if (section<0 || section>=6)
		return QVariant();

	if (role==Qt::DisplayRole){
		QString headers[]={ tr("Device"), tr("Model"), tr("Size"), tr("FS Type"), tr("Mount point"), tr("Format?") };

		return headers[section];
	}
	return QVariant();
}

/// Creates an index depending on parent and row.
QModelIndex DiskModel::index(int row, int column,
					const QModelIndex &parent) const{
	if (!parent.isValid()){
		if (disks.count()>row && row>=0){
			QModelIndex index=createIndex(row, column,(void*) disks[row]);
			return index;
		}
	}
	else{
		Node *node=NODE_FROM_INDEX(parent);
		if (node){
			if (node->partitions.count()>row){
				return createIndex(row, column,(void*) node->partitions[row]);
			}
		}
	}
	return QModelIndex();
}

/// Returns the parent for a given index.
QModelIndex DiskModel::parent(const QModelIndex &index) const{
	if (index.isValid()){
		Node *node=NODE_FROM_INDEX(index);

		if (node->parent)
			return createIndex(index.row(),index.column(),node->parent);
	}
	return QModelIndex();
}

/// Returns the number of rows: disks or partitions
int DiskModel::rowCount(const QModelIndex &model) const{
	if (!model.isValid()){
		return disks.count();
	}
	else{
		Node *node=NODE_FROM_INDEX(model);

		return node->partitions.count();
	}
	return 0;
}

/**
 * @short For all partitions, clean the format bit and mount points.
 */
void DiskModel::cleanFormatsAndMounts(){
	foreach(Node *disk, disks){
		foreach(Node *part, disk->partitions){
			part->format=false;
			part->mountPoint=QString::null;
		}
	}
	rootpartition=NULL;
	emit dataChanged(QModelIndex(),QModelIndex());
}

/// Sets the format bit. Updates model.
void DiskModel::setFormat(const QModelIndex &index, bool format){
	Node *node=NODE_FROM_INDEX(index);
	if (!node)
		return;
	node->format=format;
	emit dataChanged(index,index);
}
/// Sets the mount point. Updates model.
void DiskModel::setMountPoint(const QModelIndex &index, const QString &mountPoint){
	Node *node=NODE_FROM_INDEX(index);
	if (!node)
		return;
	node->mountPoint=mountPoint;
	if (mountPoint=="/")
		rootpartition=node;

	emit dataChanged(index,index);
}

/// Returns the partition to format as root.
QString DiskModel::getRootPartition(){
	if (rootpartition){
		if (rootpartition->partition)
			return QString("%1%2").arg(rootpartition->disk->dev->path).arg(rootpartition->partition->num);
		else
			return QString(rootpartition->disk->dev->path);
	}
	return QString::null;
}

/**
 * @short Ensures the selected partition as root is valid
 *
 * If its free space then it creates a new partition.
 */
bool DiskModel::ensureRootValid(){
	if (!rootpartition->partition){
		if (QMessageBox::question(NULL, tr("Selected disk"), 
		     tr("You selected a full disk; are sure you want to format all disk and create a new partition table?"),
			 QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Cancel){
			qDebug("%s:%d no valid partition selected. No more.",__FILE__,__LINE__);
			return false;
		}
		// I generate a new partition table, empty, and select the free space.
		qDebug("%s:%d disk %s",__FILE__,__LINE__,rootpartition->disk->dev->path);
		PedDisk *disk=ped_disk_new_fresh(rootpartition->disk->dev,ped_disk_type_get("msdos"));
		bool ok=ped_disk_commit (disk);
		qDebug("%s:%d commit %d",__FILE__,__LINE__,ok);
		
		PedPartition *partition=ped_disk_next_partition(disk,NULL);
		while (partition && partition->type!=PED_PARTITION_FREESPACE){
			qDebug("%s:%d still looking for free partition %p %d (freespace %d)",
					__FILE__,__LINE__,partition,partition->type, PED_PARTITION_FREESPACE);
			partition=ped_disk_next_partition(disk,partition);
		}
		
		if (!partition){
			QMessageBox::critical(NULL,tr("Error selecting partition"),
								  tr("The disk was repartitioned, but I was unable to select the free space as partition.\n"
								     "Try again using the Partition Manager"));
			return false;
		}
		
		Node *nrootpartition=new Node;
		nrootpartition->disk=disk;
		nrootpartition->partition=partition;
		nrootpartition->parent=rootpartition;
		nrootpartition->mountPoint="/";
		nrootpartition->format=true;
		rootpartition=nrootpartition;
	}
	
	if (rootpartition->partition->type==PED_PARTITION_FREESPACE){
		PedPartition *oldpartition=rootpartition->partition;
		
		PedPartition *partition=ped_partition_new(oldpartition->disk, 
										PED_PARTITION_NORMAL, ped_file_system_type_get("ext3"),
										oldpartition->geom.start, oldpartition->geom.end);
										
		qDebug("%s:%d partition %p",__FILE__,__LINE__,partition);
		
		//if (!partition)
		//	return false;
		
		int ok=ped_disk_add_partition(oldpartition->disk, partition, 
									  ped_constraint_any(oldpartition->disk->dev) );
		qDebug("%s:%d add_partition %d",__FILE__,__LINE__,ok);
		if (!ok)
			return false;
		
		rootpartition->partition=partition;
	}
	// for all, ensure the selected partition is marked as boot.
	int ok=ped_partition_set_flag(rootpartition->partition, PED_PARTITION_BOOT,1);
	if (!ok){
		QMessageBox::critical(NULL,tr("Error marking boot partition"),
						tr("Error marking parition as bootable. This step is needed by several broken BIOS."));
		return false;
	}
	ok=ped_disk_commit (rootpartition->disk);
	qDebug("%s:%d commit %d",__FILE__,__LINE__,ok);

	return ok;
}
