/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DISKEDIT_H__
#define __DISKEDIT_H__

#include <parted.h>

#include <QMap>
#include <QAbstractItemModel>

#include "../basemodule.h"

class QTreeView;

#define NODE_FROM_INDEX(index) (static_cast<Tuxtalador2::DiskModel::Node *>(index.internalPointer()))

namespace Tuxtalador2{

	class Process;

	/**
	 * @short Basic model for each disk node (disk or partition)
	 */
	class DiskModel : public QAbstractItemModel{
	public:
		/**
		 * @short Each of the nodes on the disk/partition tree.
		 * 
		 * If partition is NULL it is a full disk. Then format means to partition and format.
		 */
		struct Node{
			PedDisk *disk;
			PedPartition *partition;
			Node *parent;

			QString mountPoint;
			bool format;

			QList<Node*> partitions;
		};
	public:
		DiskModel();

		int rowCount(const QModelIndex &model) const;
     	int columnCount(const QModelIndex &parent = QModelIndex()) const{ return 6; };
		QVariant data(const QModelIndex &index, int role) const;
		QVariant headerData(int section, Qt::Orientation orientation,
							int role = Qt::DisplayRole) const;
		QModelIndex index(int row, int column,
						  const QModelIndex &parent = QModelIndex()) const;
		QModelIndex parent(const QModelIndex &index) const;

		void initialize();

		void cleanFormatsAndMounts();
		void setFormat(const QModelIndex &index, bool format=true);
		void setMountPoint(const QModelIndex &index, const QString &mountPoint);

		QString getRootPartition();
		bool ensureRootValid();

		Node *getNodeFromIndex(const QModelIndex &index);
	protected:
		QList<Node*> disks;
		Node *rootpartition;
	};


	/**
	 * @short Shows several pages: welcome, license and goodbye. 
	 *
	 * Nothing really important.
	 */
	class DiskEdit : public BaseModule{
	Q_OBJECT
	public:
		DiskEdit(ModuleManager *);
		~DiskEdit();

		QWidget *wizard(int step);
		QString label(int step);

		/// When at a new page, checks if it can continue.
		bool canContinueNow(int step);

	protected slots:
		void selectPartition(const QModelIndex &);
		void readLog(const QString &,QColor);
		void openPartitionManager();
		void reloadPartitionData();

	protected:
		Process *process;

		QTreeView *treeView;
		DiskModel *diskModel;
		bool mounted;
	};
}

#endif
