/*
TRANSLATOR Tuxtalador2::Userdata
*/
/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QMessageBox>
#include <QFile>
#include <QTextStream>
#include <QCheckBox>

#include <unistd.h>

#include "userdata.h"

using namespace Tuxtalador2;

Userdata::Userdata(ModuleManager *mm):BaseModule(mm){

	mm->addWizard(this, 20);
	mm->addWizard(this, 90);
}

/**
 * @short Calls prepareAskForUserdata or doSetUserdata depending on the step.
 */
QWidget *Userdata::wizard(int step){
	if (step==20)
		return prepareAskForUserdata();
	if (step==90)
		return doSetUserdata();
	return NULL;
}

/**
 * @short Prepares a form (with checking) to ask for base userdata.
 */
QWidget *Userdata::prepareAskForUserdata(){
	QWidget *root=new QWidget();

	QGridLayout *layout=new QGridLayout(root);


	QLabel *lbl=new QLabel();
	layout->addWidget(lbl,0,0,9,3);
	lbl->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	lbl->setAlignment(Qt::AlignRight|Qt::AlignBottom);

	lbl->setPixmap(QPixmap("/usr/share/icons/default.kde/128x128/apps/package_toys.png"));

	int row=0;
	

	lbl=new QLabel(tr("<h2>Introduction</h2>"
		"On this screen you must specify your own name and password,"
		" as well as root password.<h3>User data</h3>"),root);
	lbl->setWordWrap(true);
	lbl->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	layout->addWidget(lbl,row,0,1,3);

	row++;
	lbl=new QLabel(tr("Hostname:"),root);
	layout->addWidget(lbl,row,0);
	hostname=new QLineEdit(root);
	layout->addWidget(hostname,row,1,1,2);

	row++;
	lbl=new QLabel(tr("<h2>User data</h2>"
		"Enter user name and password."),root);
	lbl->setWordWrap(true);
	lbl->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	layout->addWidget(lbl,row,0,1,3);

	row++;
	lbl=new QLabel(tr("Full user name:"),root);
	layout->addWidget(lbl,row,0);
	fullname=new QLineEdit(root);
	layout->addWidget(fullname,row,1,1,2);

	row++;
	lbl=new QLabel(tr("Login:"),root);
	layout->addWidget(lbl,row,0);
	username=new QLineEdit(root);
	layout->addWidget(username,row,1,1,2);

	row++;
	lbl=new QLabel(tr("Password:"),root);
	layout->addWidget(lbl,row,0);
	password=new QLineEdit(root);
	layout->addWidget(password,row,1,1,2);
	password->setEchoMode(QLineEdit::Password);

	row++;
	lbl=new QLabel(tr("Repeat password:"),root);
	layout->addWidget(lbl,row,0);
	passwordRepeat=new QLineEdit(root);
	layout->addWidget(passwordRepeat,row,1,1,2);
	passwordRepeat->setEchoMode(QLineEdit::Password);

	
	if (manager->getConfig("desktop").isEmpty()){
		row++;
		lbl=new QLabel(tr("<h3>Root password</h3>"
			"The root password is the most important password for the system."
	" You will need it to be able to change anything on the system, such as adding"
	" new programs, hardware or users."),root);
		lbl->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
		lbl->setWordWrap(true);
		layout->addWidget(lbl,row,0,1,3);

		row++;
		lbl=new QLabel(tr("Root Password:"),root);
		layout->addWidget(lbl,row,0);
		rootPassword=new QLineEdit(root);
		layout->addWidget(rootPassword,row,1,1,2);
		rootPassword->setEchoMode(QLineEdit::Password);

		row++;
		lbl=new QLabel(tr("Root Repeat password:"),root);
		layout->addWidget(lbl,row,0);
		rootPasswordRepeat=new QLineEdit(root);
		layout->addWidget(rootPasswordRepeat,row,1,1,2);
		rootPasswordRepeat->setEchoMode(QLineEdit::Password);
		automaticLogin=NULL;
	}
	else{
		row++;
		automaticLogin=new QCheckBox(tr("Automatic login on computer turn on"), root);
		layout->addWidget(automaticLogin, row,0,1,3);

		rootPassword=NULL;
		rootPasswordRepeat=NULL;
	}


	connect(hostname,SIGNAL(textChanged(const QString &)),
			this,SLOT(textChanged(const QString &)));
	connect(fullname,SIGNAL(textChanged(const QString &)),
			this,SLOT(textChanged(const QString &)));
	connect(username,SIGNAL(textChanged(const QString &)),
			this,SLOT(textChanged(const QString &)));
	connect(password,SIGNAL(textChanged(const QString &)),
			this,SLOT(textChanged(const QString &)));
	connect(passwordRepeat,SIGNAL(textChanged(const QString &)),
			this,SLOT(textChanged(const QString &)));
	if (manager->getConfig("desktop").isEmpty()){
		connect(rootPassword,SIGNAL(textChanged(const QString &)),
				this,SLOT(textChanged(const QString &)));
		connect(rootPasswordRepeat,SIGNAL(textChanged(const QString &)),
				this,SLOT(textChanged(const QString &)));
	}
	else
		connect(automaticLogin, SIGNAL(stateChanged(int)), this, SLOT(autologinChanged(int)));
	return root;
}


/**
 * @short Saves the user/password to passwd and shadow files.
 */
QWidget *Userdata::doSetUserdata(){
	QString chroot="chroot /target ";
	if (!manager->getConfig("OEM").isEmpty())
		chroot="";
		
	QString au("%1/usr/sbin/useradd -s /bin/bash -g staff "
			   "-u 1000 -c '%2' -p %3 -m -d /home/%4 %5");
	au=au.arg(chroot).
		  arg(sfullname).arg(crypt((char*)spassword.toLatin1().data(), "AB")).
		  arg(susername).
		  arg(susername);
	emit log(au,QColor("green"));
	int result = system((char*)au.toUtf8().data());
	if (result!=0){
		QMessageBox::critical(NULL, tr("Cant create base user"), 
			tr("Error creating base user. Continuing but login may be impossible. (%1)").arg(result));
	}

	if (manager->getConfig("desktop").isEmpty()){
		au=QString("%1/usr/sbin/usermod -c '%2' -p %3 root").
				arg(chroot).
				arg(tr("System administrator")).
				arg(crypt((char*)srootPassword.toLatin1().data(), "AB"));
		emit log(au,QColor("green"));
		result=system((char*)au.toUtf8().data());
		if (result!=0){
			QMessageBox::critical(NULL, tr("Cant set root password"), 
				tr("Error setting root password. Continuing but login may be impossible. (%1)").arg(result));
		}
	}


	// add some default groups, and add the user to them
	{
		QFile f("/etc/group.default");
		f.open(QIODevice::Append);
		f.write("\nsudo\ndialout\ncdrom\nfloppy\naudio\nvideo\ngames\nlp\nlpadmin\nadm\nadmin\nnetdev\nyoucloud\n");
		f.close();
	
		f.open(QIODevice::ReadOnly);
		QTextStream stream(&f);
		QString line;
		while (!stream.atEnd())
		{
			line=stream.readLine();
			if (!line.isEmpty())
			{
				au=QString("%1addgroup %3").arg(chroot).arg(line);
				system(au.toAscii().data());
				au=QString("%1adduser %2 %3").arg(chroot).arg(susername).arg(line);
				system(au.toAscii().data());
			}
		}
	}
	au=QString("%1adduser %2 %3").arg(chroot).arg(susername).arg("sudo");
	system(au.toAscii().data());

	if (bautologin){
		emit log("Set autologin");
		manager->setConfig("AUTOLOGIN","1");
	}
	
	manager->goNext();
	return NULL;
}

QString Userdata::label(int step){
	switch(step){
	case 20:
		return tr("Setup users");
	case 90:
		return tr("Install users");
	default:
		return QString::null;
	}
}

/**
 * @short Checks if all needed data has been written, and then allows to pass
 */
void Userdata::textChanged(const QString &){
	QLineEdit *editors[]={fullname,username,password,
						 passwordRepeat,rootPassword,rootPasswordRepeat};
	bool cont=true;
	for (unsigned int i=0;i<sizeof(editors)/sizeof(editors[0]);i++){
		if (editors[i] && editors[i]->text().isEmpty())
			cont=false;
	}
	if (password->text()!=passwordRepeat->text())
		cont=false;
	
	if (manager->getConfig("desktop").isEmpty()){
		if (rootPassword->text()!=rootPasswordRepeat->text())
			cont=false;
	}
	
	if (cont){
		bautologin=(automaticLogin!=NULL) ? automaticLogin->isChecked() : false;
		sfullname=fullname->text();
		susername=username->text();
		spassword=password->text();
		if (manager->getConfig("desktop").isEmpty()){
			srootPassword=rootPassword->text();
		}
		shostname=hostname->text();
		manager->setConfig("HOSTNAME",shostname);
	}
	emit canContinue(cont);
}


/**
 * @short Jsut redirects to textChanged(...) to update status
 */
void Userdata::autologinChanged(int){
	textChanged("");
}
