/*
TRANSLATOR Tuxtalador2::Welcome
*/
/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QLabel>
#include <QTextBrowser>
#include <QLayout>
#include <QCheckBox>

#include "welcome.h"

using namespace Tuxtalador2;

/// Adds the wizards to the manager
Welcome::Welcome(ModuleManager *mm):BaseModule(mm){
	qDebug("%s:%d loaded welcome",__FILE__,__LINE__);

	mm->addWizard(this, 1);
	bool OEM=!mm->getConfig("OEM").isEmpty();
	bool firstBoot=!mm->getConfig("firstBoot").isEmpty();
	if (!OEM || (OEM && firstBoot) )
		mm->addWizard(this, 5);
	mm->addWizard(this, 100);
}

/// Shows a right wizard 
QWidget *Welcome::wizard(int step){
	QLabel *lbl=NULL;
	switch (step){
	case 1:
		qDebug("%s:%d welcome! %d",__FILE__,__LINE__,step);
		lbl=new QLabel(tr(
"<h2>Tuxum Installer</h2>"
"In the next few steps you will be asked the minimum questions about your system, so it is correctly"
" tuned for your needs."
));
		break;
	case 5:
	{
		qDebug("%s:%d welcome! %d",__FILE__,__LINE__,step);
		QWidget *license=new QWidget();
		QVBoxLayout *layout=new QVBoxLayout(license);
		QTextBrowser *browser=new QTextBrowser(license);
		browser->setSource(QUrl("file:/" TI2_DIR "/cluf.html"));
		layout->addWidget(browser);

		QCheckBox *agree=new QCheckBox(tr("I agree with the license"),license);
		layout->addWidget(agree);

		connect(agree,SIGNAL(stateChanged(int)),this,SLOT(doContinue(int)));

		license->setLayout(layout);
		return license;
		break;
	}
	case 100:
		system("eject /dev/cdrom");
		qDebug("%s:%d End! %d",__FILE__,__LINE__,step);
		lbl=new QLabel(tr("<h2>System ready</h2>"
"Your system is ready to be used. On reboot you will be asked what OS to start, and if in 10 seconds there"
" is no response, Tuxum OS will be started. <img src=\"")+ QString(TI2_DIR "/images/ok.png\"/>"));
		break;
	}
	if (lbl){
		lbl->setAlignment(Qt::AlignJustify|Qt::AlignTop);
		lbl->setWordWrap(true);
		return lbl;
	}
	return NULL;
}

/// The title page
QString Welcome::label(int step){
	switch (step){
	case 1:
		return tr("Welcome");
	case 5:
		return tr("License");
	case 100:
		return tr("Finish");
	}
	return QString::null;
}	

/// Checks if it can continue just now, or better wait for a signal.
bool Welcome::canContinueNow(int step){
	return step==1;
}

/// checks if can go back. Not if at end.
bool Welcome::canGoBack(int step){ 
	return step!=100;
}

/// emits the continue signal, if the license status is right
void Welcome::doContinue(int state){
	emit canContinue(state!=0);
}
