/*
TRANSLATOR Tuxtalador2::Process
*//**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <QMessageBox>

#include <unistd.h>

#include "basemodule.h"
#include "process.h"

using namespace Tuxtalador2;

#define QS(str) ((char*)str.toUtf8().data())

Process::Process(const QString &commandline, BaseModule *parent,
				 bool wait, bool nextOnClose) : 
	QProcess(parent),
	nextOnClose(nextOnClose),
	module(parent)
	{
	
	qDebug("%s:%d -- %s",__FILE__,__LINE__,(char*)commandline.toLatin1().data());

	connect(this, SIGNAL(readyReadStandardOutput()),
			this, SLOT(readStdin()));
	connect(this, SIGNAL(readyReadStandardError()),
			this, SLOT(readStderr()));

	connect(this,SIGNAL(log(const QString &,QColor)),
			module, SIGNAL(log(const QString &,QColor)));
	connect(this, SIGNAL(error(QProcess::ProcessError)),
			this, SLOT(errorSlot(QProcess::ProcessError)));

	if (nextOnClose)
 		connect(this, SIGNAL(finished(int , QProcess::ExitStatus )),
				this, SLOT(finishedSlot(int, QProcess::ExitStatus)));

	arguments=commandline.split(" ",QString::SkipEmptyParts);
	start(arguments[0], arguments.mid(1));

	qDebug("%s:%d %s -- %s",__FILE__,__LINE__,
		(char*)arguments[0].toLatin1().data(),
		(char*)QStringList(arguments.mid(1)).join(" ").toLatin1().data());
	
	if (nextOnClose && state()==QProcess::NotRunning)
		finished(0,QProcess::CrashExit);

	if (wait){
		while (state()!=QProcess::NotRunning){
			qApp->processEvents();
			waitForFinished(300);
		}
	}
}

void Process::readStdin(){
	QString r=readAllStandardOutput();
	//qDebug("%s:%d Read '%s'",__FILE__,__LINE__,QS(r));
	emit log(r);
}

void Process::readStderr(){
	QString err=readAllStandardError();
	qDebug("%s:%d read at stderr: %s",__FILE__,__LINE__,(char*)err.toLatin1().data());
	emit log(err,Qt::red);
}

/**
 * @short When finished checks if an error, and ask for retry, or go next.
 *
 * Only connected when nextOnClose at constructor.
 */
void Process::finishedSlot(int exitCode, QProcess::ExitStatus status){
	emit log(QString("%1 finished with code %2/%3").arg(arguments[0]).arg(exitCode).arg(status),Qt::blue);
	if (exitCode!=0 || status==QProcess::CrashExit){
		int res=QMessageBox::question(dynamic_cast<QWidget*>(parent()),tr("The process finished abnormally"),
				tr("Current step did not finish correctly. Do you want to retry it,\n",
				   "or just to terminate installation"),QMessageBox::Retry,QMessageBox::Abort);
		if (res==QMessageBox::Retry){
			sleep(1);
			qApp->processEvents();
			sleep(1);
			qApp->processEvents();
			start(arguments[0], arguments.mid(1));
		}
		//else
			//qApp->exit(1);
	}
	else{
		module->moduleManager()->goNext();
	}
}

void Process::errorSlot(QProcess::ProcessError error){
	qDebug("%s:%d error executing %s: %d",__FILE__,__LINE__,QS(arguments[0]),error);
	switch(error){
	case QProcess::FailedToStart:
		finished(0,QProcess::CrashExit);
	break;
	default:
	;
	}
}
