/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <QDesktopWidget>
#include <QTranslator>
#include <QLocale>
#include <QString>
#include <QFont>
#include <QDebug>

#include <stdio.h>

#include "modulemanager.h"
#include "moduleloader.h"
#include "wizard.h"

using namespace Tuxtalador2;


int main(int argc, char **argv){
	QApplication qapp(argc, argv);

	// Make kApp Qt-translator aware
	QTranslator translator, translatorqt;
	QString directory=QString("%1/ts/").arg(TI2_DIR);
	QString file("tuxtalador2_" + QLocale::system().name());
	translator.load( file, directory );
	qapp.installTranslator( &translator );

	translatorqt.load(QString("qt_%1.qm").arg(QLocale::system().name()),
					"/usr/share/qt4/translations/");
	qapp.installTranslator( &translatorqt );
	
	/*
	qDebug("%s:%d Loading translation file: %s%s: %s %s\n", __FILE__, __LINE__,
		directory.toAscii().data(),file.toAscii().data(),
		   ok ? "ok" : "error", translator.isEmpty() ? "empty" : "with translations");
	*/
	
	// Some options
	ModuleManager *mm=new ModuleManager();
	
	QStringList args=QApplication::arguments();
	if (args.indexOf("OEM")>0){
		mm->setConfig("OEM","1");
		if (args.indexOf("firstBoot")>0){
			mm->setConfig("firstBoot","1");
		}
	}
	if (args.indexOf("desktop")>0){
		mm->setConfig("desktop","1");
	}
	
	ModuleLoader::loadModules(mm);

	QWidget *wizard=new Wizard(mm);
	wizard->show();
	
	mm->goNext();

	wizard->showFullScreen();
	wizard->setWindowTitle(QObject::tr("Tuxum OS Instaler"));

	const int WIDTHMODIFIER = 64;
	const int HEIGHTMODIFIER = 48;

	int width = qapp.desktop()->screenGeometry().width();
	int height = qapp.desktop()->screenGeometry().height();

	int fontsize = width / WIDTHMODIFIER;

	if (fontsize > height / HEIGHTMODIFIER)
		fontsize = height / HEIGHTMODIFIER;

	QStringList fonts = QFont::substitutions();
	for (int i = 0; i < fonts.count(); i++)
	{
		qDebug() << fonts[i];
	}

	QFont defaultFont("Arial");

	defaultFont.setPixelSize(fontsize);

	qapp.setFont( defaultFont );
	
	int retcode=qapp.exec();

	delete wizard;
	delete mm;
	
	return retcode;
}
