/*
TRANSLATOR Tuxtalador2::Wizard
*/
/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QLayout>
#include <QFont>
#include <QFile>
#include <QListWidget>
#include <QPushButton>
#include <QProgressBar>
#include <QMessageBox>
#include <QApplication>
#include <QLabel>
#include <QTextEdit>
#include <QScrollBar>
#include <QAction>
#include <QInputDialog>
#include <QApplication>

#include "modulemanager.h"
#include "basemodule.h"
#include "wizard.h"

using namespace Tuxtalador2;

/**
 * @short constructs the wizard
 *
 * The wizard is set up in a grid, with a list of current actions, a mark on 
 * the current one, prev, next and cancel buttons, a title, and a main area.
 * 
 * Almost everything is obtained from BaseModules.
 */
Wizard::Wizard(ModuleManager *_moduleManager, QWidget *parent) : 
	QWidget(parent),moduleManager(_moduleManager){

	wizard=NULL;

	layout=new QGridLayout(this);

	QLabel *lbl;
	
	lbl=new QLabel(this);
	lbl->setPixmap(QPixmap(TI2_DIR "./images/watermark.png"));
	lbl->setAlignment(Qt::AlignBottom|Qt::AlignRight);
	layout->addWidget(lbl,0,0,4,6);
	
	title=new QLabel(tr("Unknown"),this);
	QFont font=title->font();
	font.setPointSize(24);
	title->setFont(font);

	layout->addWidget(title, 0,0,1,6);

	sections=new QListWidget(this);
	sections->setFrameStyle(QFrame::NoFrame);

	QListWidgetItem *li;
	foreach(ModuleManager::ModulePosition mod,moduleManager->getModules()){
		QString lbl=mod.module->label(mod.position);
		li=new QListWidgetItem(lbl, sections);
		menuitems[lbl]=li;
	}

	sections->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);
	sections->updateGeometry();

	sections->setEnabled(false);

	QPalette p;
	p.setColor(QPalette::Disabled, QPalette::Base, p.color(QPalette::Active,QPalette::Window));
	p.setColor(QPalette::Disabled, QPalette::Highlight, Qt::white);
	p.setColor(QPalette::Disabled, QPalette::HighlightedText, Qt::black);
	sections->setPalette(p);

	layout->addWidget(sections,1,0,1,1);

	QLabel *logo=new QLabel(this);
	logo->setPixmap(QPixmap(TI2_DIR "./images/logo.png"));
	layout->addWidget(logo,2,0);

	prev=new QPushButton(tr("Last"),this);
	next=new QPushButton(tr("Next"),this);
	cancel=new QPushButton(tr("Quit"),this);

	progress=new QProgressBar(this);
	progress->setRange(0,100);
	layout->addWidget(progress,4,0,1,6);

	layout->addWidget(prev,5,3,1,1);
	layout->addWidget(next,5,4,1,1);
	layout->addWidget(cancel,5,5,1,1);

	setLayout(layout);

	logPane=new QTextEdit(this);
	layout->addWidget(logPane,3,0,1,6);
	logPane->hide();


	QAction *openPane=new QAction(this);
	openPane->setShortcut(QString("F10"));
	openPane->setCheckable(true);
	openPane->setChecked(true);
	connect(openPane,SIGNAL(toggled(bool)),
			logPane, SLOT(setHidden(bool)));
	addAction(openPane);

	QAction *newTerminal=new QAction(this);
	newTerminal->setShortcut(QString("F4"));
	connect(newTerminal,SIGNAL(triggered(bool)),
			this, SLOT(openTerminal()));
	addAction(newTerminal);

	QAction *genericrun=new QAction(this);
	genericrun->setShortcut(QString("Alt+F2"));
	connect(genericrun,SIGNAL(triggered(bool)),
			this, SLOT(genericRun()));
	addAction(genericrun);

			
	QAction *nextSection=new QAction(this);
	nextSection->setShortcut(QString("Ctrl+Right"));
	connect(nextSection,SIGNAL(triggered(bool)),
			moduleManager, SLOT(goNext()));
	addAction(nextSection);

	QAction *setfullscreen=new QAction(this);
	setfullscreen->setShortcut(QString("F11"));
	connect(setfullscreen,SIGNAL(triggered(bool)),
			this, SLOT(toggleFullscreen()));
	addAction(setfullscreen);

	connect(prev,SIGNAL(clicked()), moduleManager,SLOT(goPrevious()));
	connect(next,SIGNAL(clicked()), moduleManager,SLOT(goNext()));
	connect(cancel,SIGNAL(clicked()), this,SLOT(doCancel()));

	connect(moduleManager,SIGNAL(progress(int)),
			progress, SLOT(setValue(int)));
	connect(moduleManager,SIGNAL(setWizard(BaseModule*, int)),
			this, SLOT(setWizard(BaseModule*,int)), Qt::QueuedConnection);

	connect(moduleManager,SIGNAL(log(const QString &,QColor)),
			this, SLOT(log(const QString &,QColor)));

	lbl=new QLabel(tr("<h1>Please wait</h1>\n"
"Please wait while this process ends..."));
	lbl->setWordWrap(true);
	lbl->setAlignment(Qt::AlignTop);
	defaultWizard=lbl;
	
	setMinimumSize(700,500);

}


/**
 * @short Sets the wizard page to the wizard shell.
 */
void Wizard::setWizard(BaseModule *bm, int pos){
	//qDebug("%s:%d add a new widget",__FILE__,__LINE__);
	log(tr("Show page %1").arg(pos),"#89A");

	prev->setHidden(pos==0);

	// If default wizard just hide and remove from layout. Other delete.
	if (wizard!=defaultWizard){
		delete wizard;
		wizard=defaultWizard;
	}
	else{
		wizard->hide();
		layout->removeWidget(wizard);
	}

	// If nothing here, go to the next (maybe I wanted last.. fix when it happends (FWIH))
	if (!bm){
		moduleManager->goNext();
		return;
	}

	// Gets the wizard, and if NULL use the default wizard
	wizard=bm->wizard(pos);
	if (!wizard)
		wizard=defaultWizard;
	else{
		wizard->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	
		wizard->setParent(this);
	}
	layout->addWidget(wizard,1,1,2,5);
	wizard->show();

	if (bm->canGoBack(pos))
		prev->show();
	else
		prev->hide();

	// Sets the progress point, enable If should enable and set some data.
	progress->setValue(pos);
	next->setEnabled(bm->canContinueNow(pos));

	QString lbl=bm->label(pos);

	title->setText(lbl);
	sections->clearSelection();
	menuitems[lbl]->setSelected(true);

	connect(bm,SIGNAL(canContinue(bool)),
			this,SLOT(canContinue(bool)));
}

/**
 * @short Present close/cancel dialog to prevent unwanted app close.
 */
void Wizard::doCancel(){
	if (moduleManager->atLastStep())
		QApplication::exit(0);
	else if (QMessageBox::Close==QMessageBox::question(this, tr("Sure to quit?"),
			tr("The installation is not complete yet.\n"
			   "Actually nothing have changed at your computer so it's safe to quit, but...\n"
			   "Are you sure to quit?"),QMessageBox::Close|QMessageBox::Cancel)){
		QApplication::exit(0);
	}
}

/**
 * @short Changes the next status
 */
void Wizard::canContinue(bool status){
	next->setEnabled(status);
}

/**
 * @short Adds some text to the log pane
 */
void Wizard::log(const QString &txt,QColor color){
	QString txt2=QString("<div style=\"color: %1;\">%2</div><br>").
					arg(color.name()).arg(txt.trimmed()).replace("\n","<br>");
	//qDebug("%s:%d %s",__FILE__,__LINE__,QS(txt2));
	logPane->insertHtml(txt2);
	logPane->verticalScrollBar()->setValue( logPane->verticalScrollBar()->maximum() );

	

	QFile logFile("/var/log/tuxtalador.log");

	logFile.open(QIODevice::Append);
	logFile.write( txt.toAscii() );
	logFile.close();
}


/**
 * @short Tries to open a new terminal.
 */
void Wizard::openTerminal(){
	log("Opening an xterm",Qt::blue);

	system("konsole&");
}

/**
 * @short Tries to open a new terminal.
 */
void Wizard::genericRun(){
	QString prog=QInputDialog::getText(this, tr("Select command to execute"),tr("Select command to execute"));

	log(QString("Executing '%1'").arg(prog),Qt::green);
	system(qPrintable(prog+"&"));
}


/**
 * @short Toggles the fullscreen status, usually because F11 is pressed
 */
void Wizard::toggleFullscreen(){
	log(QString("Toggle fullscreen, now %1").arg(isFullScreen() ? "fullscreen" : "normal"), Qt::green);
	if (isFullScreen())
		showNormal();
	else
		showFullScreen();
}
